String clazzTpl({package}) => """
import 'package:xanno/xanno.dart';
import 'package:flutter/material.dart';
import 'package:$package/widget/theme.dart';
import 'package:$package/generated/l10n.dart';
import 'package:$package/route/main.route.dart';
import 'package:$package/widget/observable.dart';
import 'package:$package/mixin/base_widget_mixin.dart';

class CounterState {
  int counter = 0;
}

var observableCounter = Observable(CounterState());

@GRoute(url: '/', title: 'main')
class MainPage extends BaseStatefulWidget {
  final String title;

  const MainPage({Key? key, required this.title}) : super(key: key);

  @override
  _MainPageState createState() => _MainPageState();
}

class _MainPageState extends BaseState<MainPage> with BaseStatefulMixin {
  Color? _titleBg = Color(0xff3c3f41);
  Brightness? _brightness = Brightness.dark;

  @override
  void initState() {
    observableCounter.listen((state) {
      print(state.counter);
    });
    observableTheme.listen((theme) {
      setState(() {
        _titleBg = theme.body;
        _brightness =
            theme.type == ThemeType.light ? Brightness.light : Brightness.dark;
      });
    });
    super.initState();
  }

  @override
  AppBar? appBar(BuildContext context) {
    return AppBar(
      brightness: _brightness,
      backgroundColor: _titleBg,
      title: ThemeBuilder(
        builder: (context, theme) {
          return Text(
            widget.title,
            style: TextStyle(
                fontSize: 30, fontWeight: FontWeight.bold, color: theme.title),
          );
        },
      ),
    );
  }

  @override
  Widget? body(BuildContext context) {
    return ThemeBuilder(builder: (context, theme) {
      return Container(
        color: theme.body,
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Align(
              child: ObservableBuilder<CounterState>(
                create: observableCounter,
                memo: (state) => [state.counter],
                builder: (context, state) {
                  return Text(
                    '\${S.of(context).xanno} 👍 \${state.counter}',
                    style: TextStyle(
                        fontSize: 30,
                        fontWeight: FontWeight.bold,
                        color: theme.title),
                  );
                },
              ),
            ),
            TextButton(
                style: ButtonStyle(
                    backgroundColor:
                        MaterialStateProperty.all(theme.title.withOpacity(0.2)),
                    foregroundColor: MaterialStateProperty.all(theme.title)),
                onPressed: () =>
                    Navigator.pushNamed(context, RouteConstant.AppDebugPage),
                child: Text(S.of(context).desc))
          ],
        ),
      );
    });
  }

  @override
  Widget? floatingActionButton(BuildContext context) {
    return ThemeBuilder(builder: (context, theme) {
      return FloatingActionButton(
        onPressed: () {
          observableCounter.notify((state) => state.counter++);
          // 改变主题
          observableTheme.notify((theme) => theme.type =
              theme.type == ThemeType.light ? ThemeType.dark : ThemeType.light);
        },
        backgroundColor: theme.title,
        child: Icon(Icons.add, color: theme.body),
      );
    });
  }
}
""";
