String clazzTpl({package}) => """
import 'dart:async';
import 'package:flutter/material.dart';

typedef void _Listen<T>(T state);
typedef List<dynamic> _Memo<T>(T state);
typedef Widget _Builder<T>(BuildContext context, T state);

class Observable<T> {
  final T _state;
  late StreamController<T> _controller;
  late Stream<T> _stream;
  late _Listen<T>? _listen;

  Observable(this._state) {
    _controller = StreamController.broadcast();
    _stream = _controller.stream;
    _listen = null;
  }

  Widget builder(_Builder<T> builder, {_Memo<T>? memo}) {
    return _ObservableWidget<T>(_state, _stream, builder, memo, _listen);
  }

  notify(void Function(T) fun) async {
    fun(_state);
    if (!_controller.isClosed) _controller.add(_state);
  }

  listen(_Listen<T>? listen) => _listen = listen;
}

class _ObservableWidget<T> extends StatefulWidget {
  final T state;
  final Stream<T> stream;
  final _Listen<T>? listen;
  final _Memo<T>? memo;
  final _Builder<T> builder;

  _ObservableWidget(
      this.state, this.stream, this.builder, this.memo, this.listen,
      {Key? key})
      : super(key: key);

  @override
  _ObservableWidgetState createState() =>
      _ObservableWidgetState<T>(state, stream, memo, builder, listen);
}

class _ObservableWidgetState<T> extends State<_ObservableWidget> {
  late StreamSubscription<T> _subscription;
  late List<dynamic>? _lastMemo;

  final T _state;
  final Stream<T> _stream;
  final _Listen<T>? _listen;
  final _Memo<T>? _memo;
  final _Builder<T> _builder;

  _ObservableWidgetState(
      this._state, this._stream, this._memo, this._builder, this._listen);

  @override
  void initState() {
    if (_memo != null) {
      _lastMemo = [..._memo!(_state)];
    }

    _subscription = _stream.listen((state) {
      if (_listen != null) _listen!(state);
      List<dynamic>? nowMemo;
      if (_memo != null) {
        nowMemo = [..._memo!(state)];
      }
      if (_lastMemo != null && _lastMemo!.isNotEmpty ||
          nowMemo != null && nowMemo.isNotEmpty) {
        for (var i = 0; i < _lastMemo!.length; i++) {
          if (_lastMemo![i] != nowMemo![i] && mounted) {
            _lastMemo = nowMemo;
            (context as Element).markNeedsBuild();
            break;
          }
        }
      }
    });
    super.initState();
  }

  @override
  void dispose() {
    _subscription.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return _builder(context, _state);
  }
}

class ObservableBuilder<T> extends StatelessWidget {
  const ObservableBuilder({
    Key? key,
    required this.create,
    this.memo,
    required this.builder,
  }) : super(key: key);

  final Observable<T> create;
  final _Memo<T>? memo;
  final _Builder<T> builder;

  @override
  Widget build(BuildContext context) => create.builder(builder, memo: memo);
}
""";
